--
-- SQLサンプル
--
--- Oracle Database / SQL*Plusから実行
---		プロンプトは SQL> 
--- PostgreSQL / psqlコマンドから実行
---		プロンプトは postgres=> 
--- MySQL / mysqlコマンドから実行
---		プロンプトは mysql=> 
--- SQL Server / osqlコマンドから実行
---		プロンプトは n> 

-- ２章

---- 文字列の長さ

CREATE TABLE length1(column1 CHAR(10), column2 VARCHAR(10));
INSERT INTO length1 VALUES ('𠮷', '漢 ');
SELECT LENGTH(column1), LENGTH(column2) FROM length1;


--- ゼロによる割算
SELECT 3/0;


--- エラー発生時の動作

CREATE TABLE error1(c1 INT PRIMARY KEY, c2 VARCHAR(10));
BEGIN;
INSERT INTO error1 VALUES (100, 'first');
INSERT INTO error1 VALUES (100, 'error');
INSERT INTO error1 VALUES (200, 'retry');
COMMIT;


---  GROUP BY 
SELECT District, COUNT(*) FROM city GROUP BY 1;

SET sql_mode ='';
SELECT Name, CountryCode, Count(*) FROM city;


--- 出力制限

SELECT * FROM accounts ORDER BY bid OFFSET 10 ROWS FETCH NEXT 20 ROWS ONLY;

SELECT * FROM accounts ORDER BY bid LIMIT 10, 20;


--- コメントによるヒント

EXPLAIN SELECT /*+ JOIN_ORDER(ct, cy) */ ct.Name CountryName, cy.Name CityName FROM Country ct, City cy WHERE ct.Capital = cy.ID;

CREATE TABLE hint1(col1 NUMBER PRIMARY KEY, col2 VARCHAR2(10));
INSERT INTO hint1 VALUES (100, 'first');
INSERT INTO hint1 VALUES (100, 'duplicate');
INSERT /*+ IGNORE_ROW_ON_DUPKEY_INDEX(hint1(col1)) */ INTO hint1 VALUES (100, 'duplicate');


--- 構文によるヒント

CREATE TABLE dbo.hint1(col1 INT);
SELECT COUNT(*) FROM dbo.hint1 WITH (READUNCOMMITTED) WHERE col1=100;


--- 文字列の結合

SELECT CONCAT('A', '') concat FROM DUAL;
SELECT 'A' || '' || 'B' concat FROM DUAL;


--- 暗黙の型変換

CREATE TABLE conv1 (c1 NUMBER, c2 VARCHAR2(10));
CREATE INDEX idx2_conv1 ON conv1(c2);
SELECT * FROM conv1 WHERE c2='1000';
SELECT * FROM conv1 WHERE c2=1000;


--- 数値型の計算結果
 SELECT 2/3 + 2/3 + 2/3;


-- 3章

--- 予約語
CREATE TABLE change(col1 INT);


--- データベースとスキーマ

---- PostgreSQL

CREATE SCHEMA schema1;
CREATE TABLE schema1.table1(col1 INT, col2 VARCHAR(10));
CREATE DATABASE demodb1 OWNER demo1;

---- MySQL

CREATE DATABASE demodb1 COLLATE utf8mb4_ja_0900_as_cs_ks;

---- SQL Server

CREATE SCHEMA schema1;
CREATE TABLE schema1.table1(col1 INT);


--- クラスターテーブル
CREATE TABLE dbo.Deta1 (
    DataID int NOT NULL, 
    Value1 varchar(10) NULL,
    CONSTRAINT PK_Deta1 PRIMARY KEY CLUSTERED (DataID)
) ON PRIMARY;

--- 一時テーブル

CREATE GLOBAL TEMPORARY TABLE temp1(c1 NUMBER);
CREATE PRIVATE TEMPORARY TABLE temp2(c1 NUMBER);

CREATE TEMPORARAY TABLE temp3(c1 INT);

CREATE TABLE #temp1(key1 INT, val1 VARCHAR(10));
SELECT * INTO #temp2 FROM data1;


-- マテリアライズドビュー
CREATE TABLE table1(c1 NUMBER);
CREATE MATERIALIZED VIEW mview1 REFRESH FORCE ON DEMAND AS SELECT COUNT(*) cnt FROM table1;
EXECUTE DBMS_MVIEW.REFRESH('MVIEW1', 'c');

CREATE MATERIALIZED VIEW mview1 AS SELECT COUNT(*) cnt FROM table1;
REFRESH MATERIALIZED VIEW mview1;

SET ANSI_NULLS ON;
SET QUOTED_IDENTIFIER ON;
CREATE TABLE dbo.table1(c1 INT);
CREATE VIEW view1 WITH SCHEMABINDING AS SELECT COUNT_BIG(*) cnt FROM dbo.table1;
CREATE UNIQUE CLUSTERED INDEX mview1 ON view1(cnt);


--- UNLOGGEDテーブル

CREATE TABLE unlog1(key1 NUMBER, val1 VARCHAR2(10)) NOLOGGING;

CREATE UNLOGGED TABLE unlog2(key1 NUMERIC, val1 VARCHAR(10));

CREATE TABLE mem1 (key1 INT PRIMARY KEY, val1 VARCHAR(10)) ENGINE=MEMORY;


--- 生成列

CREATE TABLE gen1(key1 INT GENERATED ALWAYS AS IDENTITY, val1 VARCHAR(10));
CREATE TABLE gen2(key1 INT GENERATED BY DEFAULT AS IDENTITY, val1 VARCHAR(10));

CREATE TABLE gen3(key1 INT PRIMARY KEY AUTO_INCREMENT, val1 VARCHAR(10));

CREATE TABLE gen4(key1 INT IDENTITY(1, 1), val1 VARCHAR(10));


--- ファンクションインデックス

SELECT * FROM `city` WHERE UPPER(`Name`) = 'TOKYO';

ALTER TABLE city ADD KEY ((UPPER(`Name`)));


--- シーケンス

CREATE SEQUENCE seq1 AS INT START WITH 100 INCREMENT BY 2 MAXVALUE 1000 CYCLE;


